@@ -4,6 +4,7 @@ namespace PHPStan\PhpDocParser\Ast\Type;
 
 use PHPStan\PhpDocParser\Ast\Node;
 use PHPStan\PhpDocParser\Ast\NodeAttributes;
+use function trim;
 
 class CallableTypeParameterNode implements Node
 {
@@ -41,7 +42,7 @@ class CallableTypeParameterNode implements Node
 		$isReference = $this->isReference ? '&' : '';
 		$isVariadic = $this->isVariadic ? '...' : '';
 		$default = $this->isOptional ? ' = default' : '';
-		return "{$type}{$isReference}{$isVariadic}{$this->parameterName}{$default}";
+		return trim("{$type}{$isReference}{$isVariadic}{$this->parameterName}") . $default;
 	}
 
 }
