@@ -279,6 +279,9 @@ class TypeParserTest extends TestCase
 						new IdentifierTypeNode('Foo'),
 						[
 							new IdentifierTypeNode('Bar'),
+						],
+						[
+							GenericTypeNode::VARIANCE_INVARIANT,
 						]
 					)
 				),
@@ -290,6 +293,10 @@ class TypeParserTest extends TestCase
 					[
 						new IdentifierTypeNode('int'),
 						new IdentifierTypeNode('Foo\\Bar'),
+					],
+					[
+						GenericTypeNode::VARIANCE_INVARIANT,
+						GenericTypeNode::VARIANCE_INVARIANT,
 					]
 				),
 			],
@@ -592,6 +599,97 @@ class TypeParserTest extends TestCase
 					),
 				]),
 			],
+			[
+				'array{a: int, b: int, ...}',
+				new ArrayShapeNode([
+					new ArrayShapeItemNode(
+						new IdentifierTypeNode('a'),
+						false,
+						new IdentifierTypeNode('int')
+					),
+					new ArrayShapeItemNode(
+						new IdentifierTypeNode('b'),
+						false,
+						new IdentifierTypeNode('int')
+					),
+				], false),
+			],
+			[
+				'array{int, string, ...}',
+				new ArrayShapeNode([
+					new ArrayShapeItemNode(
+						null,
+						false,
+						new IdentifierTypeNode('int')
+					),
+					new ArrayShapeItemNode(
+						null,
+						false,
+						new IdentifierTypeNode('string')
+					),
+				], false),
+			],
+			[
+				'array{...}',
+				new ArrayShapeNode([], false),
+			],
+			[
+				'array{
+				 *	a: int,
+				 *	...
+				 *}',
+				new ArrayShapeNode([
+					new ArrayShapeItemNode(
+						new IdentifierTypeNode('a'),
+						false,
+						new IdentifierTypeNode('int')
+					),
+				], false),
+			],
+			[
+				'array{
+					a: int,
+					...,
+				}',
+				new ArrayShapeNode([
+					new ArrayShapeItemNode(
+						new IdentifierTypeNode('a'),
+						false,
+						new IdentifierTypeNode('int')
+					),
+				], false),
+			],
+			[
+				'array{int, ..., string}',
+				new ParserException(
+					'string',
+					Lexer::TOKEN_IDENTIFIER,
+					16,
+					Lexer::TOKEN_CLOSE_CURLY_BRACKET
+				),
+			],
+			[
+				'list{
+				 	int,
+				 	string
+				 }',
+				new ArrayShapeNode(
+					[
+						new ArrayShapeItemNode(
+							null,
+							false,
+							new IdentifierTypeNode('int')
+						),
+						new ArrayShapeItemNode(
+							null,
+							false,
+							new IdentifierTypeNode('string')
+						),
+					],
+					true,
+					ArrayShapeNode::KIND_LIST
+				),
+			],
 			[
 				'callable(): Foo',
 				new CallableTypeNode(
@@ -619,6 +717,9 @@ class TypeParserTest extends TestCase
 						new IdentifierTypeNode('Foo'),
 						[
 							new IdentifierTypeNode('Bar'),
+						],
+						[
+							GenericTypeNode::VARIANCE_INVARIANT,
 						]
 					)
 				),
@@ -722,6 +823,10 @@ class TypeParserTest extends TestCase
 								[
 									new IdentifierTypeNode('mixed'),
 									new IdentifierTypeNode('string'),
+								],
+								[
+									GenericTypeNode::VARIANCE_INVARIANT,
+									GenericTypeNode::VARIANCE_INVARIANT,
 								]
 							),
 							new UnionTypeNode([
@@ -732,12 +837,19 @@ class TypeParserTest extends TestCase
 											new IdentifierTypeNode('string'),
 											[
 												new IdentifierTypeNode('foo'),
+											],
+											[
+												GenericTypeNode::VARIANCE_INVARIANT,
 											]
 										),
 										new IdentifierTypeNode('bar'),
 									])
 								),
 							]),
+						],
+						[
+							GenericTypeNode::VARIANCE_INVARIANT,
+							GenericTypeNode::VARIANCE_INVARIANT,
 						]
 					),
 					new IdentifierTypeNode('Lorem'),
@@ -769,6 +881,10 @@ class TypeParserTest extends TestCase
 						[
 							new IdentifierTypeNode('array-key'),
 							new IdentifierTypeNode('int'),
+						],
+						[
+							GenericTypeNode::VARIANCE_INVARIANT,
+							GenericTypeNode::VARIANCE_INVARIANT,
 						]
 					)
 				),
@@ -783,6 +899,10 @@ class TypeParserTest extends TestCase
 							[
 								new IdentifierTypeNode('array-key'),
 								new IdentifierTypeNode('int'),
+							],
+							[
+								GenericTypeNode::VARIANCE_INVARIANT,
+								GenericTypeNode::VARIANCE_INVARIANT,
 							]
 						)
 					),
@@ -935,6 +1055,8 @@ class TypeParserTest extends TestCase
 				'list<QueueAttributeName::*>',
 				new GenericTypeNode(new IdentifierTypeNode('list'), [
 					new ConstTypeNode(new ConstFetchNode('QueueAttributeName', '*')),
+				], [
+					GenericTypeNode::VARIANCE_INVARIANT,
 				]),
 			],
 			[
@@ -945,6 +1067,9 @@ class TypeParserTest extends TestCase
 					new IdentifierTypeNode('array'),
 					[
 						new IdentifierTypeNode('Foo'),
+					],
+					[
+						GenericTypeNode::VARIANCE_INVARIANT,
 					]
 				),
 			],
@@ -958,6 +1083,10 @@ class TypeParserTest extends TestCase
 					[
 						new IdentifierTypeNode('Foo'),
 						new IdentifierTypeNode('Bar'),
+					],
+					[
+						GenericTypeNode::VARIANCE_INVARIANT,
+						GenericTypeNode::VARIANCE_INVARIANT,
 					]
 				),
 			],
@@ -970,6 +1099,10 @@ class TypeParserTest extends TestCase
 					[
 						new IdentifierTypeNode('Foo'),
 						new IdentifierTypeNode('Bar'),
+					],
+					[
+						GenericTypeNode::VARIANCE_INVARIANT,
+						GenericTypeNode::VARIANCE_INVARIANT,
 					]
 				),
 			],
@@ -988,8 +1121,15 @@ class TypeParserTest extends TestCase
 							new IdentifierTypeNode('array'),
 							[
 								new IdentifierTypeNode('Bar'),
+							],
+							[
+								GenericTypeNode::VARIANCE_INVARIANT,
 							]
 						),
+					],
+					[
+						GenericTypeNode::VARIANCE_INVARIANT,
+						GenericTypeNode::VARIANCE_INVARIANT,
 					]
 				),
 			],
@@ -1008,8 +1148,15 @@ class TypeParserTest extends TestCase
 							new IdentifierTypeNode('array'),
 							[
 								new IdentifierTypeNode('Bar'),
+							],
+							[
+								GenericTypeNode::VARIANCE_INVARIANT,
 							]
 						),
+					],
+					[
+						GenericTypeNode::VARIANCE_INVARIANT,
+						GenericTypeNode::VARIANCE_INVARIANT,
 					]
 				),
 			],
@@ -1186,6 +1333,10 @@ class TypeParserTest extends TestCase
 						[
 							new IdentifierTypeNode('TRandKey'),
 							new IdentifierTypeNode('TRandVal'),
+						],
+						[
+							GenericTypeNode::VARIANCE_INVARIANT,
+							GenericTypeNode::VARIANCE_INVARIANT,
 						]
 					),
 					new ConditionalTypeNode(
@@ -1196,6 +1347,10 @@ class TypeParserTest extends TestCase
 							[
 								new IdentifierTypeNode('TRandKey'),
 								new IdentifierTypeNode('TRandVal'),
+							],
+							[
+								GenericTypeNode::VARIANCE_INVARIANT,
+								GenericTypeNode::VARIANCE_INVARIANT,
 							]
 						),
 						new UnionTypeNode([
@@ -1204,6 +1359,10 @@ class TypeParserTest extends TestCase
 								[
 									new IdentifierTypeNode('TRandKey'),
 									new IdentifierTypeNode('TRandVal'),
+								],
+								[
+									GenericTypeNode::VARIANCE_INVARIANT,
+									GenericTypeNode::VARIANCE_INVARIANT,
 								]
 							),
 							new GenericTypeNode(
@@ -1211,6 +1370,10 @@ class TypeParserTest extends TestCase
 								[
 									new IdentifierTypeNode('TRandKey'),
 									new IdentifierTypeNode('TRandVal'),
+								],
+								[
+									GenericTypeNode::VARIANCE_INVARIANT,
+									GenericTypeNode::VARIANCE_INVARIANT,
 								]
 							),
 						]),
@@ -1266,6 +1429,116 @@ class TypeParserTest extends TestCase
 					)
 				),
 			],
+			[
+				'(T is Foo ? true : T is Bar ? false : null)',
+				new ConditionalTypeNode(
+					new IdentifierTypeNode('T'),
+					new IdentifierTypeNode('Foo'),
+					new IdentifierTypeNode('true'),
+					new ConditionalTypeNode(
+						new IdentifierTypeNode('T'),
+						new IdentifierTypeNode('Bar'),
+						new IdentifierTypeNode('false'),
+						new IdentifierTypeNode('null'),
+						false
+					),
+					false
+				),
+			],
+			[
+				'(T is Foo ? T is Bar ? true : false : null)',
+				new ParserException(
+					'is',
+					Lexer::TOKEN_IDENTIFIER,
+					14,
+					Lexer::TOKEN_COLON
+				),
+			],
+			[
+				'($foo is Foo ? true : $foo is Bar ? false : null)',
+				new ConditionalTypeForParameterNode(
+					'$foo',
+					new IdentifierTypeNode('Foo'),
+					new IdentifierTypeNode('true'),
+					new ConditionalTypeForParameterNode(
+						'$foo',
+						new IdentifierTypeNode('Bar'),
+						new IdentifierTypeNode('false'),
+						new IdentifierTypeNode('null'),
+						false
+					),
+					false
+				),
+			],
+			[
+				'($foo is Foo ? $foo is Bar ? true : false : null)',
+				new ParserException(
+					'$foo',
+					Lexer::TOKEN_VARIABLE,
+					15,
+					Lexer::TOKEN_IDENTIFIER
+				),
+			],
+			[
+				'Foo<covariant Bar, Baz>',
+				new GenericTypeNode(
+					new IdentifierTypeNode('Foo'),
+					[
+						new IdentifierTypeNode('Bar'),
+						new IdentifierTypeNode('Baz'),
+					],
+					[
+						GenericTypeNode::VARIANCE_COVARIANT,
+						GenericTypeNode::VARIANCE_INVARIANT,
+					]
+				),
+			],
+			[
+				'Foo<Bar, contravariant Baz>',
+				new GenericTypeNode(
+					new IdentifierTypeNode('Foo'),
+					[
+						new IdentifierTypeNode('Bar'),
+						new IdentifierTypeNode('Baz'),
+					],
+					[
+						GenericTypeNode::VARIANCE_INVARIANT,
+						GenericTypeNode::VARIANCE_CONTRAVARIANT,
+					]
+				),
+			],
+			[
+				'Foo<covariant>',
+				new ParserException(
+					'>',
+					Lexer::TOKEN_CLOSE_ANGLE_BRACKET,
+					13,
+					Lexer::TOKEN_IDENTIFIER
+				),
+			],
+			[
+				'Foo<typovariant Bar>',
+				new ParserException(
+					'Bar',
+					Lexer::TOKEN_IDENTIFIER,
+					16,
+					Lexer::TOKEN_CLOSE_ANGLE_BRACKET
+				),
+			],
+			[
+				'Foo<Bar, *>',
+				new GenericTypeNode(
+					new IdentifierTypeNode('Foo'),
+					[
+						new IdentifierTypeNode('Bar'),
+						new IdentifierTypeNode('mixed'),
+					],
+					[
+						GenericTypeNode::VARIANCE_INVARIANT,
+						GenericTypeNode::VARIANCE_BIVARIANT,
+					]
+				),
+			],
 		];
 	}
 
